﻿using System;
using iiie.CacheControl.Attributes;
using iiie.CacheControl.Business.OutputCache;
using iiie.CacheControl.DBO;

namespace iiie.CacheControl.Business.Attributes
{
    /// <summary>
    /// A basic cache control based on time
    /// </summary>
    public class TimeCacheControlAttribute : CacheControlAttribute
    {
        /// <summary>
        /// The number of seconds the cache is valid
        /// </summary>
        protected int Seconds { get; set; }

        /// <summary>
        /// Contruct a cache control based on time
        /// </summary>
        /// <param name="seconds">The number of seconds the cache is valid</param>
        /// <param name="mustRevalidate">Indicates if the client can reuse cached data without asking origin server</param>
        /// <param name="excludeQueryStringFromCacheKey">Indicates if the query string must be used to control cache</param>
        /// <param name="cacheType">Define the cache type used to store cache</param>
        public TimeCacheControlAttribute(int seconds, bool mustRevalidate = true,
            bool excludeQueryStringFromCacheKey = false, OutputCacheType cacheType = OutputCacheType.Memory)
        {
            Seconds = seconds;
            MustRevalidate = mustRevalidate;
            ExcludeQueryStringFromCacheKey = excludeQueryStringFromCacheKey;
            CacheType = cacheType;
        }

        /// <summary>
        /// Check if data has been cached for less than Senconds seconds
        /// </summary>
        /// <param name="data">The cached data</param>
        /// <returns>True if cache is still valid</returns>
        protected override bool IsValid(CacheDbo data)
        {
            return data.Date.AddSeconds(Seconds) >= DateTime.Now;
        }
    }
}
